import { Controller } from '@hotwired/stimulus'

export default class TouchController extends Controller {
  static values = {
    debug: Boolean
  }

  // data-action="touchstart->slide#start:passive"
  start(event) {
    const touch = event.targetTouches[0]
    this.startPos = {
      x: touch.pageX,
      y: touch.pageY
    }
    this.startTime = new Date().getTime() // 毫秒，千分之一秒
  }

  // scale && scale !== 表示缩放了
  zoomed(event) {
    return event.changedTouches.length > 1 || (event.scale && event.scale !== 1)
  }

  // 是否达到触发条件
  effective(pad, x = true) {
    const endTime = new Date().getTime()
    let isMore
    // 滑动距离是否超过元素宽度一半
    if (x) {
      isMore = pad > this.element.clientWidth / 2
    } else {
      isMore = pad > this.element.clientHeight / 2
    }
    console.debug('是否超过一半：', isMore)

    const speed = pad / (endTime - this.startTime) // 手势速度
    const isFast = speed > 0.1
    console.debug('手势速度：', isFast) // 大于 0.1

    return isMore || isFast
  }

  offset(event) {
    const touch = event.changedTouches[0]
    const offset = {
      x: touch.pageX - this.startPos.x,
      y: touch.pageY - this.startPos.y
    }
    if (this.hasDebugValue && this.debugValue) {
      console.debug(event.type, 'offset:', offset)
    }

    return offset
  }

  get startPos() {
    let r = this.data.get('startPos').split(',')
    return {
      x: parseFloat(r[0]),
      y: parseFloat(r[1])
    }
  }

  set startPos(pos) {
    this.data.set('startPos', [pos.x, pos.y].join(','))
  }

  get startTime() {
    return this.data.get('startTime')
  }

  set startTime(time) {
    this.data.set('startTime', time)
  }

  get duration() {
    let duration = this.data.get('duration')
    if (!duration) {
      duration = '1s'
    }
    return duration
  }

}
